<?php

declare(strict_types=1);

namespace Drupal\entity_display_mode\Plugin\Field\FieldWidget;

use Psr\Log\LoggerInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;

/**
 * Define the entity display mode base widget.
 */
abstract class EntityDisplayModeBaseWidget extends WidgetBase {

  /**
   * Get the module logger instance.
   *
   * @return \Psr\Log\LoggerInterface
   */
  protected function getLogger(): LoggerInterface {
    /** @phpstan-ignore-next-line */
    return \Drupal::logger('entity_display_mode');
  }

  /**
   * Get the display mode options.
   *
   * @return array
   */
  protected function getDisplayModeOptions(): array {
    return $this->getDisplayModeTypeOptions(
      $this->fieldDefinition->getTargetEntityTypeId()
    );
  }

  /**
   * Get entity display mode type options.
   *
   * @param string $entity_type_id
   *   The entity type ID.
   *
   * @return array
   *   An array of display mode type options.
   */
  private function getDisplayModeTypeOptions(
    string $entity_type_id
  ): array {
    $display_type = $this->getFieldSetting('display_type');

    if (!isset($display_type)) {
      return [];
    }
    $options = $this->getEntityDisplayModeOptions(
      $display_type,
      $entity_type_id
    );
    $display_modes = array_filter($this->getFieldSetting('display_mode'));

    return count($display_modes) === 0
      ? $options
      : array_intersect_key($options, $display_modes);
  }

  /**
   * Get the entity display mode options.
   *
   * @param $type
   *   The display type, either form, or view.
   * @param string $entity_type_id
   *   The entity type ID.
   *
   * @return array
   *   An array of entity display mode options.
   */
  private function getEntityDisplayModeOptions(
    string $type,
    string $entity_type_id
  ): array {
    $options = [];

    switch ($type) {
      case 'form':
        $options = $this->getEntityDisplay()->getFormModeOptions(
          $entity_type_id
        );
        break;
      case 'view':
        $options = $this->getEntityDisplay()->getViewModeOptions(
          $entity_type_id
        );
        break;
    }

    return $options;
  }

  /**
   * Get entity display repository.
   *
   * @return \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   *   The entity display service.
   */
  private function getEntityDisplay(): EntityDisplayRepositoryInterface {
    /** @phpstan-ignore-next-line  */
    return \Drupal::service('entity_display.repository');
  }
}
