<?php

declare(strict_types=1);

namespace Drupal\entity_display_mode\Plugin\Field\FieldWidget;

use Drupal\Core\Utility\Error;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;

/**
 * Define the entity display mode select card widget.
 *
 * @FieldWidget(
 *   id = "entity_display_mode_select_card",
 *   label = @Translation("Select Card"),
 *   description = @Translation(
 *      "Select an entity view mode using a card output."
 *   ),
 *   field_types = {
 *     "entity_display_mode"
 *   }
 * )
 */
class EntityDisplayModeSelectCardWidget extends EntityDisplayModeBaseWidget {

  /**
   * {@inheritDoc}
   */
  public static function defaultSettings(): array {
    return [
      'display' => [],
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritDoc}
   */
  public function settingsSummary(): array {
    $summary = parent::settingsSummary();
    $summary[] = $this->t(
      'Modes: @mode',
      ['@mode' => implode(', ', array_column(
        $this->getDisplayModeDisplayDefinitions(),
        'label'
      ))]
    );

    return $summary;
  }

  /**
   * {@inheritDoc}
   */
  public function settingsForm(
    array $form,
    FormStateInterface $form_state
  ): array {
    $element = parent::settingsForm($form, $form_state);
    $display_settings = $this->getSetting('display');
    foreach ($this->getDisplayModeOptions() as $name => $label) {
      $setting = $display_settings[$name] ?? [];

      $element['display'][$name] = [
        '#type' => 'details',
        '#title' => $this->t('%label', ['%label' => $label]),
        '#open' => TRUE,
      ];
      $element['display'][$name]['label'] = [
        '#type' => 'textfield',
        '#required' => TRUE,
        '#title' => $this->t('Label'),
        '#description' => $this->t(
          'Input the label to use for the display mode.'
        ),
        '#default_value' => $setting['label'] ?? $label
      ];
      $element['display'][$name]['image_path'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Image Path'),
        '#description' => $this->t(
          'Input an image path to show the display mode representation.'
        ),
        '#default_value' => $setting['image_path'] ?? NULL,
      ];
      $element['display'][$name]['description'] = [
        '#type' => 'textarea',
        '#rows' => 2,
        '#title' => $this->t('Description'),
        '#description' => $this->t(
          'Input an explanation on when to use this display mode.'
        ),
        '#default_value' => $setting['description'] ?? NULL
      ];
    }

    return $element;
  }

  /**
   * {@inheritDoc}
   */
  public function formElement(
    FieldItemListInterface $items,
    $delta,
    array $element,
    array &$form,
    FormStateInterface $form_state
  ): array {
    $elements = [];

    /** @var \Drupal\Core\Field\FieldItemInterface|null $field_item */
    /** @phpstan-ignore-next-line */
    if ($field_item = $items[$delta] ?? NULL) {
      $elements['value'] = [
        '#type' => 'radios',
        '#options' => $this->getDisplayModeCardOptions(),
        '#wrapper_attributes' => [
          'class' => ['entity-display-mode--wrapper']
        ],
        '#default_value' => $field_item->getString()
      ] + $element;

      $elements['#attached'] = [
        'library' => [
          'entity_display_mode/card',
        ],
      ];
    }

    return $elements;
  }

  /**
   * Get display mode card options.
   *
   * @return array
   *   An array of display mode card options.
   */
  protected function getDisplayModeCardOptions(): array {
    $options = [];

    /** @var \Drupal\Core\Render\Renderer $renderer */
    /** @phpstan-ignore-next-line */
    $renderer = \Drupal::service('renderer');

    foreach ($this->getDisplayModeDisplayDefinitions() as $name => $display) {
      if (!isset($display['label'])) {
        continue;
      }
      try {
        $card = [
          '#theme' => 'entity_display_mode__card',
          '#label' => $display['label'] ?? NULL,
          '#image_path' => $display['image_path'] ?? NULL,
          '#description' => $display['description'] ?? NULL
        ];
        $options[$name] = $renderer->render($card);
      } catch (\Exception $exception) {
        Error::logException(
          $this->getLogger(),
          $exception
        );
      }
    }

    return $options;
  }

  /**
   * Get the display mode display definitions.
   *
   * @return array
   *   An array of the display mode display definitions.
   */
  protected function getDisplayModeDisplayDefinitions(): array {
    $definitions = $this->getSetting('display') ?? [];

    if (count($definitions) === 0) {
      foreach ($this->getDisplayModeOptions() as $name => $label) {
        $definitions[$name] = [
          'label' => $label,
        ];
      }
    }

    return $definitions;
  }
}
